import { useEffect } from "react";
import { makeCancelable } from "./makeCancelable";
import { usePromiseState } from "./usePromiseState";
const defaultOptions = {
    mode: "latest",
    onCancel: undefined,
};
export function usePromise(getPromise, options = defaultOptions) {
    const [state, dispatch] = usePromiseState();
    const { mode = defaultOptions.mode, onCancel = defaultOptions.onCancel } = options;
    useEffect(() => {
        if (typeof getPromise === "undefined" || getPromise === null) {
            dispatch({ type: "RESET" });
            return;
        }
        if (typeof getPromise !== "function") {
            throw new Error(`usePromise requires a function argument, but received ${typeof getPromise}`);
        }
        // Build an object containing a function to initiate the promise
        // as well as a function to cancel it. Use the customCancel arg
        // if it's provided.
        const promise = onCancel
            ? {
                getPromise,
                cancel: () => undefined,
            }
            : makeCancelable(getPromise);
        dispatch({ type: "ADD", promise });
        const instance = promise.getPromise();
        instance
            .then((result) => {
            dispatch({
                type: "COMPLETE",
                promise,
                result: { status: "SUCCESS", value: result },
            });
        })
            .catch((error) => {
            dispatch({ type: "COMPLETE", promise, result: { status: "ERROR", value: error } });
        });
        if (typeof onCancel === "function") {
            promise.cancel = () => onCancel(instance);
        }
        return () => {
            if (mode === "latest") {
                dispatch({ type: "CLEANUP", promise });
            }
        };
    }, [getPromise, dispatch, mode, onCancel]);
    return state;
}
//# sourceMappingURL=usePromise.js.map