import { useReducer } from "react";
const initialState = {
    completed: null,
    queued: [],
};
function promiseStateReducer(state = initialState, action) {
    switch (action.type) {
        case "ADD":
            return {
                ...state,
                queued: [...state.queued, action.promise],
            };
        case "CLEANUP": {
            const { promise } = action;
            if (state.queued.indexOf(promise) === -1) {
                return state;
            }
            promise.cancel();
            return {
                ...state,
                queued: state.queued.filter((queuedPromise) => queuedPromise !== promise),
            };
        }
        case "COMPLETE": {
            const queueIndex = state.queued.indexOf(action.promise);
            // If a newer promise finishes before older promises, cancel the older promises
            state.queued.slice(0, queueIndex).forEach((previousPromise) => {
                previousPromise.cancel();
            });
            return {
                ...state,
                queued: state.queued.slice(queueIndex + 1),
                completed: action.result,
            };
        }
        case "RESET":
            return initialState;
        default:
            return state;
    }
}
export function usePromiseStateReducer() {
    return useReducer(promiseStateReducer, initialState);
}
//# sourceMappingURL=usePromiseStateReducer.js.map